// staletea
// Copyright (C) 2020 Jonas Franz
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

package jobs

import "github.com/robfig/cron"

import "gitea.com/jonasfranz/staletea/config"

type Job struct {
	Interval config.Key
	Action   func() error
}

var registeredCronJobs []Job = []Job{
	{Interval: config.RepositoryScanInterval, Action: syncRepositories},
	{Interval: config.IssueScanInterval, Action: syncIssues},
}

func handleError(unsafeFunc func() error) func() {
	return func() {
		err := unsafeFunc()
		if err != nil {
			panic(err)
		}
	}
}

func StartCronJobs() error {
	c := cron.New()
	for _, job := range registeredCronJobs {
		if err := c.AddFunc(job.Interval.Get().(string), handleError(job.Action)); err != nil {
			return err
		}
	}
	c.Run()
	return nil
}
